# Generated by evpn sros.j2 template
updates:
- path: configure/router[router-name=Base]
  val:
   bgp:
    group:
{%  for type in evpn.session %}
    - group-name: {{ 'ibgp-ipv4' if type=='ibgp' else 'ebgp' }}
      family:
       evpn: True
{%  if bgp.rr|default(0) %}
      next-hop-unchanged:
       evpn: True
{%  endif %}

    rapid-withdrawal: True
    rapid-update:
     evpn: True

{%  endfor %}
