{#
  Bridge-based private network
#}
{% set ifname = defaults.providers.libvirt.vifprefix + "_" + name + "_" + l.ifindex|string %}
    {{ name }}.vm.network :private_network,
                  :libvirt__network_name => "{{ l.bridge }}",
                  :libvirt__forward_mode => "veryisolated",
                  :libvirt__dhcp_enabled => false,
{% if ifname|length < 16 %}
                  :libvirt__iface_name => "{{ ifname }}",
{% endif %}
                  :auto_config => false
