#!/bin/bash
#
# This script contains the 'ip' commands needed to set up container
# interfaces and route table. It's executed within the container
# network namespace on the container host.
#
#  /etc/hosts file is generated as a clab bind.
#
set -e
{% include 'linux/vanilla.j2' +%}
