
{% set instance = "default4_" + vname %}
{% set ospf_vrf = vname %}
{% set ospf_main_vrf = False %}
{% set ospf_router_id = vdata.ospf.router_id|default(ospf.router_id) %}

{% set ospf_afi_check = "ipv4" %}
{% set ospf_version = "2" %}
{% set ospf_interfaces = vdata.ospf.interfaces|default([]) %}
{% include 'routeros7.ospf-include.j2' %}

/routing/ospf/instance set [/routing/ospf/instance find name={{instance}}] redistribute=bgp,copy routing-table={{vname}}
