#
# Created by netlab (containerlab Linux host)
#
127.0.0.1	localhost
::1	localhost ip6-localhost ip6-loopback
fe00::0	ip6-localnet
ff00::0	ip6-mcastprefix
ff02::1	ip6-allnodes
ff02::2	ip6-allrouters
#
{% for name,node in hostvars.items() if node.af is defined and node.af.ipv4|default(False) %}
{%   if node.loopback is defined and node.loopback.ipv4 is defined %}
{{ node.loopback.ipv4.split('/')[0] }} {{ name }}
{%   elif node.interfaces and node.interfaces[0].ipv4 is defined and node.interfaces[0].ipv4 is string %}
{{ node.interfaces[0].ipv4.split('/')[0] }} {{ name }}
{%   endif %}
{% endfor %}
