#!/bin/bash

set -e

{% if ldp is defined %}
sysctl -w net.mpls.platform_labels=1048575
{%   for l in interfaces if ('ldp' in l) and not l.ldp.passive %}
sysctl -w net.mpls.conf.{{ l.ifname }}.input=1
{%   endfor %}
{% endif %}

{% include "frr.frr-config.j2" +%}

exit 0
