interface vxlan 1
  vxlan source-interface loopback 0
{% if vxlan.vlans is defined %}
{%   for vname in vxlan.vlans if vlans[vname].vni is defined %}
{%     set vlan = vlans[vname] %}
  vxlan vlan {{ vlan.id }} vni {{ vlan.vni }}
{%     if vlan.vtep_list is defined %}
  vxlan vlan {{ vlan.id }} flood vtep {{ vlan.vtep_list|join(' ') }}
{%     endif %}
{%   endfor %}
{% endif %}
