{# MPLS #}

{% if netlab_device_type|default('') == 'vsrx' %}
security {
  forwarding-options {
    family {
      mpls {
        mode packet-based;
      }
    }
  }
}
{% endif %}

{% if ldp is defined %}
{%   include 'junos.ldp.j2' +%}
{% endif %}

{% if mpls.vpn is defined %}
{%   include 'junos.mplsvpn.j2' +%}
{% endif %}
