{% set pid = ospf.process|default(0) %}
{% from "srlinux.macro.j2" import ospf_config %}
updates:
{% if ospf.af.ipv4 is defined %}
{{ ospf_config( pid,'ipv4','default',ospf,interfaces) }}
{% endif %}
{% if ospf.af.ipv6 is defined %}
{{ ospf_config( pid+1,'ipv6','default',ospf,interfaces) }}
{% endif %}
