#
# See https://documentation.nokia.com/cgi-bin/dbaccessfilename.cgi/3HE17292AAABTQZZA01_V1_7450%20ESS%207750%20SR%207950%20XRS%20MD-CLI%20Advanced%20Configuration%20Guide%20for%20Releases%20up%20to%2021.5.R2.pdf
# Proxy-ARP/ND MAC List for Dynamic Entries for a more advanced feature (L2 VPLS only)

updates:
{# Enable proxy ARP for all marked interfaces #}
{% for l in interfaces if l.vrf is defined and 'proxy_arp' in l %}
- path: configure/service/vprn[service-name={{ l.vrf }}]
  val:
   interface:
   - interface-name: {{ l.ifname }}
{%  for af in ('ipv4','ipv6') if af in l %}
     {{ af }}:
      neighbor-discovery:
       remote-proxy-arp: True
{%  endfor %}
{% endfor %}
